/*-----------------------------------------------------------------------*
 Project: [ASR Sample]
 File: [asr_sample.cpp]

 Copyright (C) 2010 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/

#include "asr_sample.h" 

#define TEST_MULTI_DIC_PATH   TEST_DATA_ROOT"dic"
#define TEST_MULTI_WAVE_PATH  TEST_DATA_ROOT"wav"
#define TEST_MULTI_DATA_PATH  TEST_DATA_ROOT"data/libASR"
#define FRAMESIZE 441


typedef struct {
    const char        *filename;
    mw::asr::DataType type;
}DataRecordT;


/* FΏی(\p) */
static const char *gLangName[D_LANG_COUNT_MAX] = {
	"Japanese",
	"British",
   	"US_English",
   	"French",
   	"Spanish",
   	"German",
   	"Italian",
    "Dutch",
    "CanadianFrench",
    "MexicanSpanish"
};

/* ˑf[^̃f(\p) */
static const char *gModelName[D_MODEL_COUNT_MAX] = {
    "mixed.large",
    "mixed.small",
    "child.large",
    "child.small"
};

/* FΏی(ݒl) */
static const mw::asr::LangType gLangType[D_LANG_COUNT_MAX] = {
    mw::asr::LANG_JAPANESE,
    mw::asr::LANG_BRITISH,
    mw::asr::LANG_US_ENGLISH,
    mw::asr::LANG_FRENCH,
    mw::asr::LANG_SPANISH,
    mw::asr::LANG_GERMAN,
    mw::asr::LANG_ITALIAN,
    mw::asr::LANG_DUTCH,
    mw::asr::LANG_CA_FRENCH,
    mw::asr::LANG_MEX_SPANISH
};

/* F̃t@C */
static const char *gDicName[D_LANG_COUNT_MAX] = {
	"Japanese100.txt",
   	"British100.txt",
	"English100.txt", 
	"French100.txt",
   	"Spanish100.txt",
   	"German100.txt", 
	"Italian100.txt",
    "Dutch100.txt",
	"CA_French100.txt",
   	"MEX_Spanish100.txt"
};

/* FΏۂ̉t@C */
static const char *gWaveFileName[D_LANG_COUNT_MAX] = {
	"JP_voice001.pcm",
	"BR_voice001.pcm",
	"US_voice001.pcm",
	"FR_voice001.pcm",
	"SP_voice001.pcm",
	"GE_voice001.pcm",
	"IT_voice001.pcm",
    "NL_voice001.pcm",
	"CA_FR_voice001.pcm",
	"MEX_SP_voice001.pcm",
};


/* ˑf[^̃t@Cƒ` */
static const DataRecordT gDataFile[] = {
    {"dtb.bin" , mw::asr::DTB_DATA_TYPE    },
    {"hmm.bin" , mw::asr::HMM_DATA_TYPE    },
    {"tree.bin", mw::asr::TREE_DATA_TYPE   },
    {"phn.bin" , mw::asr::PHN_DATA_TYPE    },
    {"mdc.bin" , mw::asr::MDC_DATA_TYPE    },
    {"str.bin" , mw::asr::STORAGE_DATA_TYPE}
};

/** Pꎫۑ̈ */
static unsigned char dict_words[ASR_MAX_DICT_SIZE+4][ASR_MAX_SENTENCE_LENGTH + 1];
/** [Nobt@ */
static unsigned char gBuff[1024];
/** ˑf[^ */
static mw::asr::DataItemType data[ASR_DATA_ITEM_NUM] ;

/** t@CpX */
static char gFilePath[2048]; // TCY傫̂ŃO[oŎ */

/** f[^ */ // TCY傫̂ŃO[oŎ */
static unsigned char gDict[ASR_MAX_DICT_SIZE * (ASR_MAX_SENTENCE_LENGTH + 7) + 4];


/************************************************************/
//  ֐
/************************************************************/
/*!--------------------------------------------------------*
 @brief  : makeLangData()
 @param  : pFilePath   I[v錾ˑf[^̃t@CpX
 @param  : pData       eˑf[^ݒ肷邽߂̗̈
 @return :  0
           s -1
 @summary: t@C( or ROM) 猾ˑf[^ǂݍ݁A
           ̈֐ݒ肷B
*----------------------------------------------------------*/
static short makeLangData(
    char                  *pFilePath,
    mw::asr::DataItemType *pData
){
   short i;
   FILE *fp;

   char path[256];
   MEMSET( path, 0, sizeof(path)/sizeof(path[0]));


   for(i=0; i<ASR_DATA_ITEM_NUM; i++){
      pData[i].type = gDataFile[i].type ;
      sprintf(path, "%s/%s", pFilePath, gDataFile[i].filename);

      fp = FILE_OPEN(path, "rb") ;

      if(fp == NULL){
          goto cleanup ;
      }
      pData[i].size = FILE_GETSIZE(fp) ;
      // Y.Sakurai 2005.01.19 
      if (pData[i].size == 0) {
          pData[i].data = NULL;
      } else  {
          pData[i].data = OS_MALLOC32( pData[i].size) ;
          if(pData[i].data == NULL){
              goto cleanup ;
          }
//          if(FILE_READ(pData[i].data, sizeof(char), pData[i].size, fp) != D_MATH_ROUNDUP32(pData[i].size)){
          long ret = FILE_READ(pData[i].data, sizeof(char), pData[i].size, fp);
          if( ret != (long)((pData[i].size))){
              PRINT_OUT( "ret   = %d\n", ret );
              PRINT_OUT( "len   = %d\n",pData[i].size);
              PRINT_OUT( "32len = %d\n",D_MATH_ROUNDUP32(pData[i].size));

              goto cleanup ;
          }
      }
      FILE_CLOSE(fp);
    }

    return 0 ;

cleanup :
    if(fp){
        FILE_CLOSE(fp) ;
    }
    for(i=0; i<ASR_DATA_ITEM_NUM; i++){
        if(pData[i].data){
            OS_FREE(pData[i].data) ;
        }
    }
    return -1 ;
}
/*!--------------------------------------------------------*
 @brief  : makeDict()
 @param  : pFilePath    I[v鎫̃t@CpX
 @param  : pDict        f[^
 @return :  0
           s -1
 @summary: t@C( or ROM) 猾ˑf[^ǂݍ݁A
           ̈֐ݒ肷B
 @note   : Pꎫt@C̃pXw肵AioOeLXgԂ
            pDict  [MAX_DICT_SIZE * (MAX_SENTENCE_LENGTH + 7) + 4] TCY
           wt@C1s1PLq(SAMPȀꍇ͐擪'#'t)
*----------------------------------------------------------*/
short makeDict(
    char          *pFilePath,
    unsigned char *pDict
){

   FILE *pf = NULL;
   int n, n_words = 0;
   unsigned char *p;

   char path[256];
   MEMSET( path, 0, sizeof(path)/sizeof(path[0]));

   pf = FILE_OPEN (pFilePath, "rb");
   if (!pf) {
       PRINT_OUT("[Error] Could to open file \"%s\" for input\n", pFilePath);
       return -1;
   }

   n_words = 0;
   MEMSET( gBuff,0, sizeof(gBuff));

   while (FILE_FGETS ((char*)gBuff, 1024, pf)){
      if (strlen ((char*)gBuff)){
          p = gBuff + strlen ((char*)gBuff) - 1;
          while (p >= gBuff && (*p == '\n' || *p == ' ' || *p == '\t' || *p == '\r')){
              *(p--) = 0;
          }
      }
      if (gBuff[0]) {
          MEMCPY ((char*)dict_words[n_words], (char*)gBuff,sizeof(char) * (strlen ((char*)gBuff) + 1));
          n_words++;
      }
   }

   p = pDict;
   for (n = 0; n < n_words; n++){
       p += sprintf ((char*)p, "%d:%s", n + 1, (char*)dict_words[n]) + 1;
   }
   sprintf ((char*)p, "END");

   FILE_CLOSE (pf);

   return n_words;
}

/*!--------------------------------------------------------*
 @brief  : getDictWord()
 @param  : index     Index
 @return : Yindex̎
*----------------------------------------------------------*/
unsigned char* getDictWord(int index)
{
    if( index < 0 || index > ASR_MAX_DICT_SIZE ){
        return NULL;
	}
	return dict_words[index];
}
/************************************************************/
//  OJ֐
/************************************************************/

/*!--------------------------------------------------------*
 brief  : getLangName()
 param  : idx   Ώیindex
 return : FΏی(\p)
 summary: inx Ŏw肵F̕Ԃ
*----------------------------------------------------------*/
char* getLangName(int idx){
    if( idx < 0 || idx >= D_LANG_COUNT_MAX ){
        PRINT_OUT("[ERROR] getLangName index=%d\n",idx );
        return NULL;
    }
    return (char*)gLangName[idx];
}

/*!--------------------------------------------------------*
 @brief  : getModelName()
 @param  : idx   Ώیindex
 @return : ˑf[^̃f(\p) 
 @summary: 
*----------------------------------------------------------*/
char* getModelName(int idx){
    if( idx < 0 || idx >= D_MODEL_COUNT_MAX ){
        PRINT_OUT("[ERROR] getModelName index=%d\n",idx );
        return NULL;
    }
    return (char*)gModelName[idx];
}
/*!--------------------------------------------------------*
 @brief  : getModelNumMax()
 @param  : idx   Ώیindex
 @return : Ȃ
 @summary: ꃂf̍ő吔擾B
*----------------------------------------------------------*/
int getModelNumMax( int idx ){

    int ret = 4;
    mw::asr::LangType lang;

    if( idx < 0 || idx >= D_LANG_COUNT_MAX ){
        PRINT_OUT("[ERROR] getModelNumMax index=%d\n",idx );
        return ret;
    }

    lang = gLangType[idx];

    switch( lang ){
    case mw::asr::LANG_JAPANESE:
    case mw::asr::LANG_BRITISH:
    case mw::asr::LANG_US_ENGLISH:
    case mw::asr::LANG_FRENCH:
    case mw::asr::LANG_SPANISH:
    case mw::asr::LANG_GERMAN:
    case mw::asr::LANG_ITALIAN:
        ret = 4;
        break;
    case mw::asr::LANG_CHINESE:
        ret = 1;
        break;
    case mw::asr::LANG_DUTCH:
    case mw::asr::LANG_KOREAN:
    case mw::asr::LANG_RUSSIAN:
    case mw::asr::LANG_CA_FRENCH:
    case mw::asr::LANG_MEX_SPANISH:
        ret = 2;
        break;
    default:
        PRINT_OUT("[ERROR] getModelNumMax index=%d lang=%d \n",idx, lang );
        break;
    }
    
    return ret;

}
 
/*!--------------------------------------------------------*
 @brief  : asr_test()
 @param  : aLangID
 @param  : aModel
 @return :  :0
         : s :-1
 @summary: F̃eXg֐
*----------------------------------------------------------*/
int asr_test( int aLangID,int aModel )
{
    s16 ret, i, k;
    s16 rcnt;
    mw::asr::LangType lang;
    s16 *workMem ;
    void *dataitem;
    s32 workMemSize;
    FILE *wavfp = NULL;

    // Fʎ擾p
    s16 candiWord ;
    s16 candiScore ;

    u16 frameSize;
    s32 fileSize;
    s32 wavSize;

#ifdef ENABLE_TIMESCALE    
    OS_TICK             tx[2];
#endif /* #ifdef ENABLE_TIMESCALE */

    s16* framePtr=NULL;
    s16* startPtr=NULL;

    // 2005.07.13 Y.Sakurai
    int count = 0;
    s16 AsrRecogStartedFlag = 0;
    int frameCnt=0;
    int started=0;

#ifdef ENABLE_TIMESCALE    
    tx[0]=tx[1]=0;
#endif /* #ifdef ENABLE_TIMESCALE */

    for (i = 0; i < ASR_DATA_ITEM_NUM; i++){
        data[i].data = NULL;
    }
        
    workMemSize = mw::asr::GetWorkMemSize() ;
    PRINT_OUT("workMemSize = %d\n", workMemSize) ;
    workMem = (s16 *)OS_MALLOC(workMemSize) ;
    dataitem = (void*)OS_MALLOC(100);
    if(workMem == NULL || dataitem == NULL){
        goto cleanup;
    }

    PRINT_OUT("Select Language=%s\n", gLangName[aLangID]) ;

    lang = gLangType[aLangID];

    PRINT_OUT("Exec %s-%s  ...\n", gLangName[aLangID], gModelName[aModel]) ;

    // ǂݍ
    sprintf(gFilePath, "%s/%s", TEST_MULTI_DIC_PATH, gDicName[aLangID]);
    ret = makeDict(gFilePath, gDict);
    if (ret < 0) {
        PRINT_OUT("[ERROR]: MakeDict failed.\n") ;
        goto cleanup;
    }
    PRINT_OUT("MakeDic Success\n");

    // ˑf[^ǂݍ
    sprintf(gFilePath, "%s/%s/%s", TEST_MULTI_DATA_PATH, gLangName[aLangID], gModelName[aModel]);
    ret = makeLangData(gFilePath, data);
    if (ret != 0) {
        PRINT_OUT("[ERROR]: MakeLangData failed.\n") ;
        goto cleanup;
    }
    PRINT_OUT("MakeLangData Success\n");

#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    ret = mw::asr::RecogInit(lang, data, ASR_DATA_ITEM_NUM, workMem) ;
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("  RecogInit() : %ld %s \n", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#endif /* #ifdef ENABLE_TIMESCALE */

    if(ret != ASR_NO_ERROR){
        PRINT_OUT("[ERROR]:%s-%s RecogInit Failed. ret = %d\n", gLangName[aLangID], gModelName[aModel], ret) ;
        goto cleanup;
    }

#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    ret = mw::asr::SetDict(gDict, 100);
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("  SetDict() : %ld %s \n", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#endif /* #ifdef ENABLE_TIMESCALE */

    if(ret != ASR_NO_ERROR){
        PRINT_OUT("[ERROR]:%s-%s SetDict Failed. ret = %d\n", gLangName[aLangID], gModelName[aModel], ret) ;
        goto cleanup;
    }

    // eXgJn
    // wav t@Cǂݍ
    sprintf(gFilePath, "%s/%s", TEST_MULTI_WAVE_PATH, gWaveFileName[aLangID]);
    wavfp = FILE_OPEN(gFilePath, "rb");
    if (wavfp == NULL) {
        PRINT_OUT("[ERROR]:%s-%s WavFile open Failed. ret = %d\n", gLangName[aLangID], gModelName[aModel], ret) ;
        goto cleanup;
    }


    /* wavt@C̃TCY擾 */
    fileSize = (s32)FILE_GETSIZE(wavfp);

    /* TCŸ̗m */
    framePtr = (s16*)OS_MALLOC32(fileSize);
    if (framePtr == NULL) {
        PRINT_OUT("Failed. malloc\n");
        goto cleanup;
    }
    MEMSET32(framePtr, 0, fileSize);

    /* wavt@C̓eSēǂݍ */
//    wavSize = FILE_READ(framePtr, sizeof(s16),fileSize/sizeof(s16), wavfp);
    wavSize = FILE_READ(framePtr, 1, fileSize, wavfp);
    //if (wavSize != D_MATH_ROUNDUP32(fileSize)) {
    if (wavSize != fileSize) {
        PRINT_OUT("Failed. read file\n");
        goto cleanup;
    }

    // FJn
#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    ret = mw::asr::RecogStart() ;
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("  RecogStart() : %ld %s \n", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#endif /* #ifdef ENABLE_TIMESCALE */

    startPtr = framePtr;
    AsrRecogStartedFlag = 0;
    count = 0;

#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

    while(ret == ASR_NO_ERROR){

        if (((s32)fileSize - ((s32)FRAMESIZE*2)) >= 0) {
            frameSize = FRAMESIZE;
            fileSize -= (FRAMESIZE*2); 
        } else if( fileSize <= 0 ){
            PRINT_OUT("  FILE END!!\n");
            frameSize = 0;
        } else {
            frameSize = (u16)(fileSize/2);
            fileSize -= fileSize; 
        }

        ret = mw::asr::RecogFrame(framePtr, frameSize) ;
        if(ret != ASR_NO_ERROR){
            // 2005.07.13 Y.Sakurai V1.0.3 dlύX
            if (ret == ASR_RECOG_STARTED) {
                if (AsrRecogStartedFlag == 0) {        
                    PRINT_OUT("    ASR_RECOG_STARTED at count = %d\n", count);
#ifdef ENABLE_TIMESCALE
// Zbg
//tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

                    AsrRecogStartedFlag = 1;
                    started=count;
                }
                ret = ASR_NO_ERROR;
            }
        }
        count++;


        if(fileSize <= 0) {
//            PRINT_OUT("  End of File\n");
//            break;
            continue;
        } else {
            framePtr  += FRAMESIZE;
        }

    }
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("  RecogFrame() : %ld %s\n", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#endif /* #ifdef ENABLE_TIMESCALE */

    frameCnt =  (int)(count-started);

    PRINT_OUT("\n===========================================\n");
    PRINT_OUT("[Recognized:] (%d), %d frames \n", ret, frameCnt) ;
    for(rcnt=1; rcnt<=ret; rcnt++){
        candiWord = mw::asr::GetCandidateWord(rcnt) ;
        candiScore = mw::asr::GetCandidateScore(rcnt) ;
        PRINT_OUT("  [%d] %d:%s (%d) \n", rcnt, candiWord, getDictWord(candiWord-1), candiScore );
    }
    PRINT_OUT("===========================================\n\n");

    if(wavfp){
        FILE_CLOSE(wavfp) ;
        wavfp = NULL;
    }
    if( framePtr ){
        framePtr=startPtr;
        OS_FREE(framePtr);
        framePtr=NULL;
    }
#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    ret = mw::asr::RecogEnd(dataitem) ;
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("  RecogEnd() : %ld %s \n", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#endif /* #ifdef ENABLE_TIMESCALE */

    if (ret != 100) {
            PRINT_OUT("[ERROR]: RecogEnd failed %d.\n", ret) ;
            goto cleanup;
    }

    // f[^J
    for(k=0; k<ASR_DATA_ITEM_NUM; k++){
        if(data[k].data){
            OS_FREE(data[k].data) ;
            data[k].data = NULL;
        }
    }


    PRINT_OUT("%s-%s Finished\n", gLangName[aLangID], gModelName[aModel]) ;

    for(i=0; i<ASR_DATA_ITEM_NUM; i++){
        if(data[i].data){
            OS_FREE(data[i].data) ;
            data[i].data=NULL;
        }
    }
    if(workMem){
        OS_FREE(workMem) ;
        workMem = NULL;
    }
    if(dataitem){
        OS_FREE(dataitem) ;
        dataitem = NULL;
    }
    PRINT_OUT("--------------------------\n") ;
    PRINT_OUT("Application End (Success)\n");
    return 0 ;

cleanup:

#ifdef ENABLE_TIMESCALE
tx[0] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */
    ret = mw::asr::RecogEnd(dataitem) ;
#ifdef ENABLE_TIMESCALE
tx[1] = OS_GET_TICK();
#endif /* #ifdef ENABLE_TIMESCALE */

#ifdef ENABLE_TIMESCALE
    PRINT_OUT("  RecogEnd() : %ld %s \n", OS_TICKS_TO_MSEC(tx[1]-tx[0]), TIME_UNIT);
#endif /* #ifdef ENABLE_TIMESCALE */

    for(i=0; i<ASR_DATA_ITEM_NUM; i++){
        if(data[i].data){
            OS_FREE(data[i].data) ;
            data[i].data = NULL;
        }
    }
    if(workMem){
        OS_FREE(workMem) ;
        workMem = NULL;
    }
    if(dataitem){
        OS_FREE(dataitem) ;
        dataitem=NULL;
    }

    if(wavfp){
        FILE_CLOSE(wavfp) ;
        wavfp=NULL;
    }

    PRINT_OUT("--------------------------\n") ;
    PRINT_OUT("Application End (Failed)\n");

    return -1 ;
}
