/*-----------------------------------------------------------------------*
 Project: [ASR GuiSample]
 File: [asr_font:.cpp]

 Copyright (C) 2010 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/
/******************************************************************
                        include t@C
*******************************************************************/
#include <nn/os.h>
#include <nn/fs.h>
#include <nn/font.h>

#include "CTR_Define.h"

using namespace nn::os;
using namespace nn::fs;
using namespace nn::font;


/******************************************************************
            OJ֐
*******************************************************************/
//---------------------------------------------------------------------------
//! @brief      VF[_̏s܂B
//!
//! @param[in,out]  pResource `p\[XǗIuWFNgւ̃|C^B
//---------------------------------------------------------------------------
void*
InitShaders(nn::font::RectDrawer* pDrawer)
{
    const wchar_t* shaders = L"rom:/shbin/nwfont_RectDrawerShader.shbin";

    nn::fs::FileReader shaderReader(shaders);

    const u32 fileSize = (u32)shaderReader.GetSize();

    void* shaderBinary = OS_MALLOC(fileSize);
    NN_NULL_ASSERT(shaderBinary);

#ifndef NN_BUILD_RELEASE
    s32 read =
#endif // NN_BUILD_RELEASE
    shaderReader.Read(shaderBinary, fileSize);
    NN_ASSERT(read == fileSize);

    const u32 vtxBufCmdBufSize =
        nn::font::RectDrawer::GetVertexBufferCommandBufferSize(shaderBinary, fileSize);
    void *const vtxBufCmdBuf = OS_MALLOC(vtxBufCmdBufSize);
    NN_NULL_ASSERT(vtxBufCmdBuf);
    pDrawer->Initialize(vtxBufCmdBuf, shaderBinary, fileSize);

    OS_FREE(shaderBinary);

    return vtxBufCmdBuf;

}


//---------------------------------------------------------------------------
//! @brief      `̏ݒs܂B
//!
//! @param[in]  width           ʂ̕B
//! @param[in]  height          ʂ̍B
//---------------------------------------------------------------------------
void
InitDraw(
    int     width,
    int     height
)
{
    // J[obt@
    // LCĎɍ킹āAƍւĂ܂B
    const nn::font::ColorBufferInfo colBufInfo = { width, height, PICA_DATA_DEPTH24_STENCIL8_EXT };

    const u32 screenSettingCommands[] =
    {

        // r[|[g̐ݒ
        NN_FONT_CMD_SET_VIEWPORT( 0, 0, colBufInfo.width, colBufInfo.height ),

        // VU[𖳌
        NN_FONT_CMD_SET_DISABLE_SCISSOR( colBufInfo ),

        // wobt@̖
        // fvXW̐ݒ
        // |SItZbg̖
        NN_FONT_CMD_SET_WBUFFER_DEPTHRANGE_POLYGONOFFSET(
            1.0f,           // wScale : 0.0 Wobt@
            0.0f,           // depth range near
            1.0f,           // depth range far
            0,              // polygon offset units : 0.0  |SItZbg
            colBufInfo),
    };

    nngxAdd3DCommand(screenSettingCommands, sizeof(screenSettingCommands), true);

    static const u32 s_InitCommands[] =
    {
        // JO𖳌
        NN_FONT_CMD_SET_CULL_FACE( NN_FONT_CMD_CULL_FACE_DISABLE ),

        // XeVeXg𖳌
        NN_FONT_CMD_SET_DISABLE_STENCIL_TEST(),

        // fvXeXg𖳌
        // J[obt@̑SĂ̐݉
        NN_FONT_CMD_SET_DEPTH_FUNC_COLOR_MASK(
            false,  // isDepthTestEnabled
            0,      // depthFunc
            true,   // depthMask
            true,   // red
            true,   // green
            true,   // blue
            true),  // alpha

        // A[[fvXeXg𖳌
        NN_FONT_CMD_SET_ENABLE_EARLY_DEPTH_TEST( false ),

        // t[obt@ANZX
        NN_FONT_CMD_SET_FBACCESS(
            true,   // colorRead
            true,   // colorWrite
            false,  // depthRead
            false,  // depthWrite
            false,  // stencilRead
            false), // stencilWrite
    };

    nngxAdd3DCommand(s_InitCommands, sizeof(s_InitCommands), true);
}

//---------------------------------------------------------------------------
//! @brief      ResFont\z܂B
//!
//! @param[out] pFont    \ztHgւ̃|C^B
//! @param[in]  filePath [htHg\[Xt@CB
//!
//! @return     ResFont\z̐ۂԂ܂B
//---------------------------------------------------------------------------
bool
InitFont(
    nn::font::ResFont*  pFont,
    const wchar_t*      filePath
)
{
    // tHg\[X[h܂
    nn::fs::FileReader fontReader(filePath);

    s32 fileSize = (s32)fontReader.GetSize();
    if ( fileSize <= 0 )
    {
        return false;
    }

    void* buffer = OS_MALLOC_EX(fileSize, nn::font::GlyphDataAlignment);
    if (buffer == NULL)
    {
        return false;
    }

    s32 readSize = fontReader.Read(buffer, fileSize);
    if (readSize != fileSize)
    {
        OS_FREE(buffer);
        return false;
    }

    // tHg\[XZbg܂
    bool bSuccess = pFont->SetResource(buffer);
    NN_ASSERT(bSuccess);

    //--- Ƀ\[XZbgς݂ł邩C[hς݂ł邩A\[XsȏꍇɎs܂B
    if (! bSuccess)
    {
        OS_FREE(buffer);
    }

    // `pobt@ݒ肵܂B
    const u32 drawBufferSize = nn::font::ResFont::GetDrawBufferSize(buffer);
    void* drawBuffer = OS_MALLOC_EX(drawBufferSize, 4);
    NN_NULL_ASSERT(drawBuffer);
    pFont->SetDrawBuffer(drawBuffer);

    return bSuccess;
}

//---------------------------------------------------------------------------
//! @brief      ResFontj܂B
//!
//! @param[in]  pFont           jtHgւ̃|C^B
//---------------------------------------------------------------------------
void
CleanupFont(nn::font::ResFont* pFont)
{
    // `pobt@̖
    // `pobt@ZbgĂȂ \z SetDrawBuffer ɓnobt@ւ
    // |C^ԂĂ܂B
    void *const drawBuffer = pFont->SetDrawBuffer(NULL);
    if (drawBuffer != NULL)
    {
        OS_FREE(drawBuffer);
    }

    // tHgZbgĂȂ SetResource ɓn\[Xւ
    // |C^ԂĂ܂B
    void *const resource = pFont->RemoveResource();
    if (resource != NULL)
    {
        OS_FREE(resource);
    }

    // RemoveResource ͍ēx SetResource ܂ŃtHgƂĎgpł܂B
}

//---------------------------------------------------------------------------
//! @brief      \pobt@mۂ܂B
//!
//! @param[in]  charMax         \镶̍ő啶B
//!
//! @return     mۂ\pobt@ւ̃|C^Ԃ܂B
//---------------------------------------------------------------------------
nn::font::DispStringBuffer*
AllocDispStringBuffer(int charMax)
{
    const u32 DrawBufferSize = nn::font::CharWriter::GetDispStringBufferSize(charMax);
    void *const bufMem = OS_MALLOC(DrawBufferSize);
    NN_NULL_ASSERT(bufMem);

    return nn::font::CharWriter::InitDispStringBuffer(bufMem, charMax);
}

//---------------------------------------------------------------------------
//! @brief      \pɃfr[sƎˉesݒ肵܂B
//!
//! @param[in]  pDrawer         RectDrawerIuWFNgւ̃|C^B
//! @param[in]  width           ʂ̕B
//! @param[in]  height          ʂ̍B
//---------------------------------------------------------------------------
void
SetupTextCamera(
    nn::font::RectDrawer*   pDrawer,
    int                     width,
    int                     height
)
{
    // ˉes𐳎ˉeɐݒ
    {
        // ㌴_ƂAYŽtɂȂ悤ɐݒ肵܂B
        nn::math::MTX44 proj;
        f32 znear   =  0.0f;
        f32 zfar    = -1.0f;
        f32 t       =  0;
        f32 b       =  static_cast<f32>(width);
        f32 l       =  0;
        f32 r       =  static_cast<f32>(height);
        nn::math::MTX44OrthoPivot(&proj, l, r, b, t, znear, zfar, nn::math::PIVOT_UPSIDE_TO_TOP);
        pDrawer->SetProjectionMtx(proj);
    }

    // fr[sPʍsɐݒ
    {
        nn::math::MTX34 mv;
        nn::math::MTX34Identity(&mv);
        pDrawer->SetViewMtxForText(mv);
    }
}

//---------------------------------------------------------------------------
//! @brief      ASCII`悵܂B
//!
//! @param[in]  pDrawer         RectDrawerIuWFNgւ̃|C^B
//! @param[in]  pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^B
//! @param[in]  pFont           tHgւ̃|C^B
//! @param[in]  width           ʂ̕B
//! @param[in]  height          ʂ̍B
//---------------------------------------------------------------------------
void
DrawTextWithFontLong(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    char                            pStrList[10][128],
    int                             num,
    int                             width,
    int                             height
)
{
    int cnt;
    nn::font::TextWriter writer;
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetCursor(0, 0);

    pDrawer->DrawBegin();

    SetupTextCamera(pDrawer, width, height);

    // JE^̕`R}h̓sx쐬Ďgp܂B
    writer.StartPrint();
    for( cnt=0;cnt<num;cnt++){
        if( pStrList[cnt] != NULL ){
            (void)writer.Printf(pStrList[cnt]);
        }
    }
    writer.EndPrint();
    pDrawer->BuildTextCommand(&writer);
    writer.UseCommandBuffer();

    pDrawer->DrawEnd();
}


//---------------------------------------------------------------------------
//! @brief      ASCII`悵܂B
//!
//! @param[in]  pDrawer         RectDrawerIuWFNgւ̃|C^B
//! @param[in]  pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^B
//! @param[in]  pFont           tHgւ̃|C^B
//! @param[in]  width           ʂ̕B
//! @param[in]  height          ʂ̍B
//---------------------------------------------------------------------------
void
DrawLevelMeter(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    int                             level
)
{
    int cnt;
    int max;
    char buf[41];
    nn::util::Color8 color( nn::util::Color8::RED );

    nn::font::TextWriter writer;
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetCursor(0,NN_GX_DISPLAY1_HEIGHT/2);
    writer.SetTextColor( color );
    writer.SetScale(1.0f);

    level/=100;   // max̓}CN̎dl250ƂȂ͂
    level/=2;

    max=level;
    if( max > 40){
        max = 40;
    }

    MEMSET(buf,0,sizeof(buf));
    for(cnt=0;cnt<max;cnt++){
        sprintf( &buf[cnt],"|");
    }

    pDrawer->DrawBegin();

    SetupTextCamera(pDrawer, NN_GX_DISPLAY1_WIDTH, NN_GX_DISPLAY1_HEIGHT);

    writer.StartPrint();
    (void)writer.Printf(buf);
    writer.EndPrint();
    pDrawer->BuildTextCommand(&writer);
    writer.UseCommandBuffer();

    pDrawer->DrawEnd();
}

//---------------------------------------------------------------------------
//! @brief      ASCII`悵܂B
//!
//! @param[in]  pDrawer         RectDrawerIuWFNgւ̃|C^B
//! @param[in]  pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^B
//! @param[in]  pFont           tHgւ̃|C^B
//! @param[in]  width           ʂ̕B
//! @param[in]  height          ʂ̍B
//---------------------------------------------------------------------------
void
DrawTextWithFont(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    char*                           pString,
    int                             width,
    int                             height
)
{
    nn::font::TextWriter writer;
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetCursor(0, 0);

    pDrawer->DrawBegin();

    SetupTextCamera(pDrawer, width, height);

    writer.StartPrint();
    (void)writer.Printf(pString);
    writer.EndPrint();
    pDrawer->BuildTextCommand(&writer);
    writer.UseCommandBuffer();

    pDrawer->DrawEnd();
}

//---------------------------------------------------------------------------
//! @brief      ASCII`悵܂B
//!
//! @param[in]  pDrawer         RectDrawerIuWFNgւ̃|C^B
//! @param[in]  pDrawStringBuf  DispStringBufferIuWFNgւ̃|C^B
//! @param[in]  pFont           tHgւ̃|C^B
//! @param[in]  width           ʂ̕B
//! @param[in]  height          ʂ̍B
//---------------------------------------------------------------------------
void
DrawText(
    nn::font::RectDrawer*           pDrawer,
    nn::font::DispStringBuffer*     pDrawStringBuf,
    nn::font::ResFont*              pFont,
    char*                           pString,
    int                             x,
    int                             y,
    int                             width,
    int                             height
)
{
    nn::font::TextWriter writer;
    writer.SetDispStringBuffer(pDrawStringBuf);
    writer.SetFont(pFont);

    writer.SetCursor(x, y);

    pDrawer->DrawBegin();

        SetupTextCamera(pDrawer, width, height);

        writer.StartPrint();
        (void)writer.Printf(pString);
        writer.EndPrint();
        pDrawer->BuildTextCommand(&writer);
        writer.UseCommandBuffer();

    pDrawer->DrawEnd();
}

