/*-----------------------------------------------------------------------*
 Project: [ASR GuiSample]
 File: [asr_mic.cpp]

 Copyright (C) 2010 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/

#include <nn/mic.h>
#include "asr_mic.h"
#include "CTR_Define.h"

/*!--------------------------------------------------------*
 @brief  : Initialize()
 @param  : Ȃ
 @return : Ȃ
 @summary: }CN֘Ȁ
           SDKmic̃Tv炻̂܂ܗ
*----------------------------------------------------------*/
void MicDemo::Initialize( void* aMicBuffer, int aBufferSize, bool* aSleepTime )
{
    size_t size;
    nn::Result result;

    // mic ̏
    result = nn::mic::Initialize();
    NN_UTIL_PANIC_IF_FAILED(result);

    // }CNCuɃobt@ݒ
    result = nn::mic::SetBuffer(aMicBuffer, (size_t)aBufferSize);
    NN_UTIL_PANIC_IF_FAILED(result);

    m_MicBuffer = (s16*)aMicBuffer;

    // TvOobt@TCY擾
    result = nn::mic::GetSamplingBufferSize(&size);
    NN_UTIL_PANIC_IF_FAILED(result);
    m_SamplingLength = size / sizeof(s16);

    // MIC̏Ԃ擾
    result = nn::mic::GetAmp( &m_AmpStatus );
    NN_UTIL_PANIC_IF_FAILED(result);

    if( m_AmpStatus ){
        // }CNON̏ꍇAQCۑĂݒ
        result = nn::mic::GetAmpGain(&m_MicGain);
        NN_UTIL_PANIC_IF_FAILED(result);
        *aSleepTime = false; // 50msec
    }else{
        // }CNOFF̏ꍇAONʃXbh𗧂ĂSleep
        result = nn::mic::SetAmp( true );
        NN_UTIL_PANIC_IF_FAILED(result);
        *aSleepTime = true; // 500msec
    }

    // }CNQC̐ݒ
    result = nn::mic::SetAmpGain(MIC_GAIN);
    NN_UTIL_PANIC_IF_FAILED(result);

    // }CNNvLɐݒ
    result = nn::mic::SetClamp(true);
    NN_UTIL_PANIC_IF_FAILED(result);


    // }CNdONAX[v畜A͂Pb
    // }CN͂肵Ȃ IɖɂĂ邽߃AvSleepő҂B
}

/*!--------------------------------------------------------*
 @brief  : Finalize()
 @param  : Ȃ
 @return : Ȃ
 @summary: }CN֘ȀI
           SDKmic̃Tv炻̂܂ܗ
*----------------------------------------------------------*/
void MicDemo::Finalize()
{
    nn::Result result;

    // TvO̒~
    result = nn::mic::StopSampling();
    if ( result.IsFailure() ) {
        if ( result != nn::mic::CTR::ResultShellClose() ) {
            NN_UTIL_PANIC_IF_FAILED(result);
        }
    }

    // ̏Ԃɖ߂(dAQC)
    if ( !m_AmpStatus) {
        // }CNdOFF@c̏ꍇQCݒ͕sv
        result = nn::mic::SetAmp(false);
        NN_UTIL_PANIC_IF_FAILED(result);

    }else{
        // }CNQCɖ߂
        if (m_MicGain != MIC_GAIN) {
            result = nn::mic::SetAmpGain(m_MicGain);
            NN_UTIL_PANIC_IF_FAILED(result);
        }
    }

    // }CNCuɐݒ肵obt@gȂ悤ɂ
    result = nn::mic::ResetBuffer();
    NN_UTIL_PANIC_IF_FAILED(result);

    result = nn::mic::Finalize();
    NN_UTIL_PANIC_IF_FAILED(result);


}

/*!--------------------------------------------------------*
 @brief  : Start()
 @param  : Ȃ
 @return : Ȃ
 @summary: }CN֘A TvOJn
*----------------------------------------------------------*/
void MicDemo::Start()
{
    ASR_ReadStart = reinterpret_cast<s16*>(m_MicBuffer);
    m_lastSampAddress = reinterpret_cast<uptr>(m_MicBuffer);

    // }CNTvOJn
    nn::Result result;
    result = nn::mic::StartSampling(nn::mic::SAMPLING_TYPE_SIGNED_16BIT,
                                    nn::mic::SAMPLING_RATE_10910,
                                    0,                                   // ItZbg
                                    m_SamplingLength * sizeof(s16),      // TCY
                                    true);
    if ( result.IsFailure() )
    {
        if ( result != nn::mic::CTR::ResultShellClose() )
        {
            NN_UTIL_PANIC_IF_FAILED(result);
        }
    }
            
    m_micLoop=0;
}
/*!--------------------------------------------------------*
 @brief  : Stop()
 @param  : Ȃ
 @return : Ȃ
 @summary: }CN֘A TvOI
*----------------------------------------------------------*/
void MicDemo::Stop()
{
    // }CNTvOI
    nn::Result result;
    // ŃTvO̒~sĂAmic::Finalize ɎŒ~
    result = nn::mic::StopSampling();
    if ( result.IsFailure() )
    {
        if ( result != nn::mic::CTR::ResultShellClose() )
        {
            NN_UTIL_PANIC_IF_FAILED(result);
        }
    }
}


/*!--------------------------------------------------------*
 @brief  : GetMicData()
 @param  : pDest   t[obt@̈
 @param  : pSize   擾TCY
 @return : Ȃ
 @summary: }CN֘A TvOʂ̎擾(ϒ)
*----------------------------------------------------------*/
void MicDemo::GetMicData(s16** pDest, u32* pSize)
{
    uptr pData=0;
    nn::Result result = nn::mic::GetLastSamplingAddress(&pData);
    NN_UTIL_PANIC_IF_FAILED(result);
    // Jgt[̐擪AhX=PÕt[̍ŏIAhX
    s16* data = reinterpret_cast<s16*>(m_lastSampAddress);
    u32 len = static_cast<u32>(pData - m_lastSampAddress) / 2; // in WORD

    *pDest = data;
    *pSize = len;
    m_lastSampAddress = pData;
}

/*!--------------------------------------------------------*
 @brief  : ReadBuffer()
 @param  : pFrame     t[obt@ւ̃|C^
 @return : 擾t[TCY
 @summary: }CN֘A TvOʂ̎擾(Œ蒷)
 @note   : XbhȂƒxB
*----------------------------------------------------------*/
s16 MicDemo::ReadBuffer( s16 *pFrame )
{
    s16 *readEnd = ASR_ReadStart + FRAME_SIZE;
    s16 *last;
    s16 *Top;
    s16 *Bottom;
    u32 size = (FRAME_SIZE) << 1; // Byte ̃TCY
    s16 ret;
    uptr pData=0;

    nn::Result result = nn::mic::GetLastSamplingAddress(&pData);
    NN_UTIL_PANIC_IF_FAILED(result);

    Top  = reinterpret_cast<s16*>(m_MicBuffer);
    Bottom = Top + (MIC_BUFFER_SIZE >> 1); //  s16 ̃TCY
    last = reinterpret_cast<s16*>(pData);
        
    // MutexLock
//    m_mutex.Lock();

#ifdef ASR_DEBUG
    PRINT_OUT("0x%x ", ASR_ReadStart);
#endif
    if (!m_micLoop) {
        // obt@ւ̃Cg|C^܂ԂĂȂ
        if (readEnd <= last) {
            // t[f[^荞
            MEMCPY((void *)pFrame, (const void *)ASR_ReadStart, size);
            // startXV
            ASR_ReadStart = readEnd;
            ret = 1;
            goto unlock_exit;
        } else {
            ret = 0;
            goto unlock_exit;
        }
    } else {
        // Cg|C^܂Ԃ
        if (readEnd <= Bottom) {
            // start - end Ԃ荞
            MEMCPY((void *)pFrame, (const void *)ASR_ReadStart, size);
            // startXV
            ASR_ReadStart = readEnd;
            ret = 1;
            goto unlock_exit;
        } else {
            // end  Bottom zĂ
            readEnd = Top + (readEnd - Bottom);
            if (readEnd <= last) {
                u8 *cont;
                // start - Bottom Ԃ荞
                size = (u32)((u8 *)Bottom - (u8 *)ASR_ReadStart);
                if (size) {
                    MEMCPY((void *)pFrame, (const void *)ASR_ReadStart, size);
                }
                // Top - end Ԃ荞
                cont = (u8 *)pFrame + size;
                size = (u32)((u8 *)readEnd - (u8 *)Top);
                if (size) {
                    MEMCPY((void *)cont, (const void *)Top, size);
                }

                // ܂ԂI
                m_micLoop = 0;
                ASR_ReadStart = readEnd;
                ret = 1;
                goto unlock_exit;
            } else {
                ret = 0;
                goto unlock_exit;
            }
        }
    }

    unlock_exit:

    // MutexUnlock
//    m_mutex.Unlock();
   return ret;
}

/*---------------------------------------------------------------------------*
  End of file
 *---------------------------------------------------------------------------*/
