/*-----------------------------------------------------------------------*
 Project: [ASR GuiSample]
 File: [asr_mw.cpp]

 Copyright (C) 2010 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/

#include "asr_mw.h" 

#define TEST_MULTI_DIC_PATH   TEST_DATA_ROOT"dic"
#define TEST_MULTI_WAVE_PATH  TEST_DATA_ROOT"wav"
#define TEST_MULTI_DATA_PATH  TEST_DATA_ROOT"data/libASR"

// t[TCY
#define ASR_FRAME_SIZE 128

#define REC_BUFFER_SIZE      (60*1000*10910/ASR_FRAME_SIZE*2)

typedef struct {
    const char        *filename;
    mw::asr::DataType type;
}DataRecordT;


/* FΏی(\p) */
static const char *gLangName[D_LANG_COUNT_MAX] = {
    "Japanese",
    "British",
    "US_English",
    "French",
    "Spanish",
    "German",
    "Italian",
    "Dutch",
    "CanadianFrench",
    "MexicanSpanish"
};

/* ˑf[^̃f(\p) */
static const char *gModelName[D_MODEL_COUNT_MAX] = {
    "mixed.large",
    "mixed.small",
    "child.large",
    "child.small"
};

/* FΏی(ݒl) */
static const mw::asr::LangType gLangType[D_LANG_COUNT_MAX] = {
    mw::asr::LANG_JAPANESE,
    mw::asr::LANG_BRITISH,
    mw::asr::LANG_US_ENGLISH,
    mw::asr::LANG_FRENCH,
    mw::asr::LANG_SPANISH,
    mw::asr::LANG_GERMAN,
    mw::asr::LANG_ITALIAN,
    mw::asr::LANG_DUTCH,
    mw::asr::LANG_CA_FRENCH,
    mw::asr::LANG_MEX_SPANISH
};

/* F̃t@C */
static const char *gDicName[D_LANG_COUNT_MAX] = {
    "Japanese100.txt",
    "British100.txt",
    "English100.txt", 
    "French100.txt",
    "Spanish100.txt",
    "German100.txt", 
    "Italian100.txt",
    "Dutch100.txt",
    "CA_French100.txt",
    "MEX_Spanish100.txt"
};

/* ˑf[^̃t@Cƒ` */
static const DataRecordT gDataFile[] = {
    {"dtb.bin" , mw::asr::DTB_DATA_TYPE    },
    {"hmm.bin" , mw::asr::HMM_DATA_TYPE    },
    {"tree.bin", mw::asr::TREE_DATA_TYPE   },
    {"phn.bin" , mw::asr::PHN_DATA_TYPE    },
    {"mdc.bin" , mw::asr::MDC_DATA_TYPE    },
    {"str.bin" , mw::asr::STORAGE_DATA_TYPE}
};

/** Pꎫۑ̈ */
static unsigned char dict_words[ASR_MAX_DICT_SIZE+4][ASR_MAX_SENTENCE_LENGTH + 1];
/** [Nobt@ */
static unsigned char gBuff[1024];


/************************************************************/
//  OJ֐
/************************************************************/
/*!--------------------------------------------------------*
 @brief  : makeLangData()
 @param  : pFilePath   I[v錾ˑf[^̃t@CpX
 @param  : pData       eˑf[^ݒ肷邽߂̗̈
 @return :  0
           s -1
 @summary: t@C( or ROM) 猾ˑf[^ǂݍ݁A
           ̈֐ݒ肷B
*----------------------------------------------------------*/
short makeLangData(
    char         *pFilePath,
    DataItemType *pData
){
    short i;
    FILE *fp;

    char path[256];
    MEMSET( path, 0, sizeof(path)/sizeof(path[0]));


    for(i=0; i<ASR_DATA_ITEM_NUM; i++){
       pData[i].type = gDataFile[i].type ;
       sprintf(path, "%s/%s", pFilePath, gDataFile[i].filename);

       fp = FILE_OPEN(path, "rb") ;

       if(fp == NULL){
           goto cleanup ;
       }
       pData[i].size = FILE_GETSIZE(fp) ;
       // Y.Sakurai 2005.01.19 
       if (pData[i].size == 0) {
           pData[i].data = NULL;
       } else  {
           pData[i].data = OS_MALLOC32(pData[i].size) ;
           if(pData[i].data == NULL){
               goto cleanup ;
           }
           if(FILE_READ(pData[i].data, sizeof(char), pData[i].size, fp) != D_MATH_ROUNDUP32(pData[i].size)){
               goto cleanup ;
           }
       }
       FILE_CLOSE(fp);
     }

     return 0 ;

cleanup :
     if(fp){
         FILE_CLOSE(fp) ;
     }
     for(i=0; i<ASR_DATA_ITEM_NUM; i++){
         if(pData[i].data){
             OS_FREE(pData[i].data) ;
             pData[i].data=NULL;
         }
     }
     return -1 ;
}

/*!--------------------------------------------------------*
 @brief  : deleteLangData()
 @param  : pData       eˑf[^ݒ肷邽߂̗̈
 @return : Ȃ
 @summary: ˑf[^J
*----------------------------------------------------------*/
void deleteLangData(
    DataItemType *pData
){
    short i;

    if( pData == NULL ){
        return;
    }

    for(i=0; i<ASR_DATA_ITEM_NUM; i++){
       if( pData[i].data ){
           OS_FREE(pData[i].data);
           pData[i].data=NULL;
       }
    }

    return;
}

/*!--------------------------------------------------------*
 @brief  : makeDict()
 @param  : pFilePath    I[v鎫̃t@CpX
 @param  : pDict        f[^
 @return :  0
           s -1
 @summary: t@C( or ROM) 猾ˑf[^ǂݍ݁A
           ̈֐ݒ肷B
 @note   : Pꎫt@C̃pXw肵AioOeLXgԂ
            pDict  [MAX_DICT_SIZE * (MAX_SENTENCE_LENGTH + 7) + 4] TCY
           wt@C1s1PLq(SAMPȀꍇ͐擪'#'t)
*----------------------------------------------------------*/
short makeDict(
    char          *pFilePath,
    unsigned char *pDict
){

    FILE *pf = NULL;
    int n, n_words = 0;
    unsigned char *p;

    char path[256];
    MEMSET( path, 0, sizeof(path)/sizeof(path[0]));

    pf = FILE_OPEN (pFilePath, "rb");
    if (!pf) {
        PRINT_OUT("[Error] Could to open file \"%s\" for input\n", pFilePath);
        return -1;
    }

    n_words = 0;
    MEMSET( gBuff,0, sizeof(gBuff));

    while (FILE_FGETS ((char*)gBuff, 1024, pf)){
       if (strlen ((char*)gBuff)){
           p = gBuff + strlen ((char*)gBuff) - 1;
           while (p >= gBuff && (*p == '\n' || *p == ' ' || *p == '\t' || *p == '\r')){
               *(p--) = 0;
           }
       }
       if (gBuff[0]) {
           MEMCPY ((char*)dict_words[n_words], (char*)gBuff,sizeof(char) * (strlen ((char*)gBuff) + 1));
           n_words++;
       }
    }

    p = pDict;
    for (n = 0; n < n_words; n++){
        p += sprintf ((char*)p, "%d:%s", n + 1, (char*)dict_words[n]) + 1;
    }
    sprintf ((char*)p, "END");

    FILE_CLOSE (pf);

    return n_words;
}

/*!--------------------------------------------------------*
 @brief  : getDictWord()
 @param  : index     Index
 @return : Yindex̎
*----------------------------------------------------------*/
unsigned char* getDictWord(int index)
{
    if( index < 0 || index > ASR_MAX_DICT_SIZE ){
        return NULL;
    }
    return dict_words[index];
}

/*!--------------------------------------------------------*
 brief  : getLangType()
 param  : idx   Ώیindex
 return : LangType
 summary: inx Ŏw肵F̕Ԃ
*----------------------------------------------------------*/
LangType getLangType(int idx){
    if( idx < 0 || idx >= D_LANG_COUNT_MAX ){
        PRINT_OUT("[ERROR] getLangName index=%d\n",idx );
        return gLangType[0];
    }
    return gLangType[idx];
}


/*!--------------------------------------------------------*
 brief  : getLangName()
 param  : idx   Ώیindex
 return : FΏی(\p)
 summary: inx Ŏw肵F̕Ԃ
*----------------------------------------------------------*/
char* getLangName(int idx){
    if( idx < 0 || idx >= D_LANG_COUNT_MAX ){
        PRINT_OUT("[ERROR] getLangName index=%d\n",idx );
        return NULL;
    }
    return (char*)gLangName[idx];
}

/*!--------------------------------------------------------*
 @brief  : getDicName()
 @param  : idx   Ώیindex
 @return : 
 @summary: 
*----------------------------------------------------------*/
char* getDicName(int idx){
    if( idx < 0 || idx >= D_LANG_COUNT_MAX ){
        PRINT_OUT("[ERROR] getDicName index=%d\n",idx );
        return NULL;
    }
    return (char*)gDicName[idx];
}
/*!--------------------------------------------------------*
 @brief  : getModelName()
 @param  : idx   Ώیindex
 @return : ˑf[^̃f(\p) 
 @summary: 
*----------------------------------------------------------*/
char* getModelName(int idx){
    if( idx < 0 || idx >= D_MODEL_COUNT_MAX ){
        PRINT_OUT("[ERROR] getModelName index=%d\n",idx );
        return NULL;
    }
    return (char*)gModelName[idx];
}
/*!--------------------------------------------------------*
 @brief  : getModelNumMax()
 @param  : idx   Ώیindex
 @return : Ȃ
 @summary: ꃂf̍ő吔擾B
*----------------------------------------------------------*/
int getModelNumMax( int idx ){

    int ret = 4;
    mw::asr::LangType lang;

    if( idx < 0 || idx >= D_LANG_COUNT_MAX ){
        PRINT_OUT("[ERROR] getModelNumMax index=%d\n",idx );
        return ret;
    }

    lang = gLangType[idx];

    switch( lang ){
    case mw::asr::LANG_JAPANESE:
    case mw::asr::LANG_BRITISH:
    case mw::asr::LANG_US_ENGLISH:
    case mw::asr::LANG_FRENCH:
    case mw::asr::LANG_SPANISH:
    case mw::asr::LANG_GERMAN:
    case mw::asr::LANG_ITALIAN:
        ret = 4;
        break;
    case mw::asr::LANG_CHINESE:
        ret = 1;
        break;
    case mw::asr::LANG_DUTCH:
    case mw::asr::LANG_KOREAN:
    case mw::asr::LANG_RUSSIAN:
    case mw::asr::LANG_CA_FRENCH:
    case mw::asr::LANG_MEX_SPANISH:
        ret = 2;
        break;
    default:
        PRINT_OUT("[ERROR] getModelNumMax index=%d lang=%d \n",idx, lang );
        break;
    }
    
    return ret;

}
