// ----------------------------------------------------------------------
// OtBbNNX
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include    <nn.h>
#include    <nn/os.h>
#include	<nn/gx.h>
#include	<nn/math.h>

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CtrlGraphics.h"
#include	"CTR_Define.h"

// ----------------------------------------------------------------------
// CCtrlGraphics::CCtrlGraphics
// ----------------------------------------------------------------------
// e
//   RXgN^
// ----------------------------------------------------------------------
// 
//   
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
CCtrlGraphics::CCtrlGraphics()
{
	m_PgID					= 0;
	m_ShID					= 0;
	m_HeapForGx				= NULL;

	m_textureWidth			= 0;
	m_textureHeight			= 0;
	m_textureGLDataBuffer	= NULL;
	m_textureDataBuffer		= NULL;
}

// ----------------------------------------------------------------------
// CCtrlGraphics::~CCtrlGraphics
// ----------------------------------------------------------------------
// e
//   fXgN^
// ----------------------------------------------------------------------
// 
//   
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
CCtrlGraphics::~CCtrlGraphics()
{
	FinalizeGraphics();
}

// ----------------------------------------------------------------------
// CCtrlGraphics::ClearDisplay
// ----------------------------------------------------------------------
// e
//   w肳ꂽfBXvC̃NA[s܂B
// ----------------------------------------------------------------------
// 
//   desplay		[ I ] fBXvCw肵܂B
//                        NN_GX_DISPLAY0 㑤
//                        NN_GX_DISPLAY1 
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::ClearDisplay(const s32 display)
{
	m_RenderSystemDrawing.SetRenderTarget(display);
	m_RenderSystemDrawing.Clear();
}

// ----------------------------------------------------------------------
// CCtrlGraphics::SwapBuffer
// ----------------------------------------------------------------------
// e
//   w肳ꂽfBXvC̍XVs܂B
//   {֐Ă΂ꂽ_ł͂߂ĕ`悳܂B
// ----------------------------------------------------------------------
// 
//   desplay		[ I ] fBXvCw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::SwapBuffer(const s32 display)
{
	m_RenderSystemDrawing.SetRenderTarget(display);
	m_RenderSystemDrawing.SwapBuffers();
}

// ----------------------------------------------------------------------
// CCtrlGraphics::InitializeGraphics
// ----------------------------------------------------------------------
// e
//   OtBbN̏s܂B
// ----------------------------------------------------------------------
// 
//   
// ----------------------------------------------------------------------
// ߂l
//   0Ԃ܂B
// ----------------------------------------------------------------------
u32 CCtrlGraphics::InitializeGraphics()
{
	m_HeapForGx = (uptr)OS_MALLOC(0x400000);

	/* Initialize display */
	m_RenderSystemDrawing.Initialize(m_HeapForGx, 0x400000);

	m_PgID = glCreateProgram();
	m_ShID = glCreateShader(GL_VERTEX_SHADER);

	// t@CVXeɂVF[_oCi̓ǂݍ
//	const size_t ROMFS_BUFFER_SIZE = 1024 * 64;
//    static char buffer[ROMFS_BUFFER_SIZE];

//	nn::fs::MountRom(16, 16, buffer, ROMFS_BUFFER_SIZE);
//	nn::fs::FileReader shaderReader(L"rom:/shader.shbin");
	nn::fs::FileReader shaderReader(L"rom:/shbin/shader.shbin");

	size_t size = shaderReader.GetSize();
	void* pShader = OS_MALLOC(size);

	s32 read = shaderReader.Read(pShader, size);
	NN_ASSERT(read == size);
	glShaderBinary(1, &m_ShID, GL_PLATFORM_BINARY_DMP, pShader, read);

	glAttachShader(m_PgID, m_ShID);
	glAttachShader(m_PgID, GL_DMP_FRAGMENT_SHADER_DMP);

	glBindAttribLocation(m_PgID, 0, "aPosition");
	glBindAttribLocation(m_PgID, 1, "aColor");

	glLinkProgram(m_PgID);
	glValidateProgram(m_PgID);
	glUseProgram(m_PgID);

	OS_FREE((void*)pShader);

	m_RenderSystemDrawing.SetClearColor(NN_GX_DISPLAY0, 0.0f, 0.0f, 1.0f, 1.0f);
//	m_RenderSystemDrawing.SetClearColor(NN_GX_DISPLAY1, 0.f, 0.f, 0.f, 1.0f);
	m_RenderSystemDrawing.SetClearColor(NN_GX_DISPLAY1, 0.36f, 0.42f, 0.5f, 1.0f);

	glClearDepthf(1.f);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glFrontFace(GL_CCW);

	glUniform3i(glGetUniformLocation(m_PgID, "dmp_TexEnv[0].srcRgb"), GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);
	glUniform3i(glGetUniformLocation(m_PgID, "dmp_TexEnv[0].srcAlpha"), GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);

	return 0;
}

// ----------------------------------------------------------------------
// CCtrlGraphics::FinalizeGraphics
// ----------------------------------------------------------------------
// e
//   OtBbN̔js܂B
// ----------------------------------------------------------------------
// 
//   
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::FinalizeGraphics()
{
	m_RenderSystemDrawing.Finalize();

	if (m_HeapForGx != NULL)
	{
		OS_FREE((void*)m_HeapForGx);
		m_HeapForGx = NULL;
	}

	if (m_textureGLDataBuffer != NULL)
	{
		OS_FREE(m_textureGLDataBuffer);
		m_textureGLDataBuffer = NULL;
	}

	if (m_textureDataBuffer != NULL)
	{
		OS_FREE(m_textureDataBuffer);
		m_textureDataBuffer = NULL;
	}
}

// ----------------------------------------------------------------------
// CCtrlGraphics::SetColor
// ----------------------------------------------------------------------
// e
//   `悷Fw肵܂B
// ----------------------------------------------------------------------
// 
//   desplay		[ I ] fBXvCw肵܂B
//   r				[ I ] RGBRw肵܂B(1.0 ` 0.0)
//   g				[ I ] RGBGw肵܂B(1.0 ` 0.0)
//   b				[ I ] RGBBw肵܂B(1.0 ` 0.0)
// ----------------------------------------------------------------------
// ߂l
//   0Ԃ܂B
// ----------------------------------------------------------------------
void CCtrlGraphics::SetColor(const s32 display, GLfloat r, GLfloat g, GLfloat b)
{
	m_RenderSystemDrawing.SetRenderTarget(display);

	m_RenderSystemDrawing.SetColor(r, g, b);
}

// ----------------------------------------------------------------------
// CCtrlGraphics::DrawLine
// ----------------------------------------------------------------------
// e
//   w肳ꂽfBXvC̎w肳ꂽʒuɐ`悵܂B
// ----------------------------------------------------------------------
// 
//   desipaly			[ I ] fBXvCw肵܂B
//   sx					[ I ] `JnXWw肵܂B
//   sy					[ I ] `JnYWw肵܂B
//   ex					[ I ] `IXWw肵܂B
//   ey					[ I ] `IYWw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   0Ԃ܂B
// ----------------------------------------------------------------------
void CCtrlGraphics::DrawLine(const s32 display, f32 sx, f32 sy, f32 ex, f32 ey)
{
	m_RenderSystemDrawing.SetRenderTarget(display);

	m_RenderSystemDrawing.SetLineWidth(2.0f);

	m_RenderSystemDrawing.DrawLine(sx, sy, ex, ey);
}

// ----------------------------------------------------------------------
// CCtrlGraphics::DrawRect
// ----------------------------------------------------------------------
// e
//   w肳ꂽfBXvC̎w肳ꂽʒuɎlp`悵܂B
// ----------------------------------------------------------------------
// 
//   desipaly			[ I ] fBXvCw肵܂B
//   sx					[ I ] `JnXWw肵܂B
//   sy					[ I ] `JnYWw肵܂B
//   ex					[ I ] `IXWw肵܂B
//   ey					[ I ] `IYWw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   0Ԃ܂B
// ----------------------------------------------------------------------
void CCtrlGraphics::DrawRect(const s32 display, f32 sx, f32 sy, f32 ex, f32 ey)
{
	m_RenderSystemDrawing.SetRenderTarget(display);

	DrawLine(display, sx, sy, ex, sy);
	DrawLine(display, sx, ey, ex, ey);
	DrawLine(display, sx, sy, sx, ey);
	DrawLine(display, ex, sy, ex, ey);
}

// ----------------------------------------------------------------------
// CCtrlGraphics::DrawFull
// ----------------------------------------------------------------------
// e
//   w肳ꂽfBXvC̎w肳ꂽʒuhԂ܂B
// ----------------------------------------------------------------------
// 
//   desipaly			[ I ] fBXvCw肵܂B
//   sx					[ I ] `JnXWw肵܂B
//   sy					[ I ] `JnYWw肵܂B
//   ex					[ I ] `IXWw肵܂B
//   ey					[ I ] `IYWw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   0Ԃ܂B
// ----------------------------------------------------------------------
void CCtrlGraphics::DrawFull(const s32 display, f32 sx, f32 sy, f32 ex, f32 ey)
{
	m_RenderSystemDrawing.SetRenderTarget(display);
	m_RenderSystemDrawing.FillRectangle(sx, sy, ex - sx + 1, ey - sy + 1);
}

// ----------------------------------------------------------------------
// GetTextureLength
// ----------------------------------------------------------------------
// e
//   œKȃeNX`[TCY擾܂B
//   eNX`[GetTextureLengthŎ擾TCYoȂΓ
//     Ȃ
// ----------------------------------------------------------------------
// 
//   imageLength		[I/O] eNX`[Ɏw肷摜̃TCYw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   œKȃTCYԂ܂B
// ----------------------------------------------------------------------
u32 GetTextureLength(const u32& imageLength)
{        
	u32 textureLength = 8;

	// 8, 16, 32, 64, 128, 256, 512, 1024
	for (u32 i = 0; i < 7; i++)
	{
		if ( imageLength > textureLength )
		{
			textureLength *= 2;
		}
		else
		{
			return textureLength;
		}
	}
	return 1024;
}

// ----------------------------------------------------------------------
// CCtrlGraphics::InitTextureBuffer
// ----------------------------------------------------------------------
// e
//   eNX`[obt@̏s܂B
//   ݒ肵ĂTCYƈႤꍇɂ̂݃obt@̍Ď擾s܂B
// ----------------------------------------------------------------------
// 
//   textureWidth		[ I ] eNX`[̕w肵܂B
//   textureHeight		[ I ] eNX`[̍w肵܂B
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::InitTextureBuffer(u32 textureWidth, u32 textureHeight)
{
	// ݂̃eNX`[TCYƔr
	if (textureWidth == m_textureWidth && textureHeight == m_textureHeight)
	{
		return;
	}

	// GLData
	if (m_textureGLDataBuffer != NULL)
	{
		OS_FREE(m_textureGLDataBuffer);
	}
	m_textureGLDataBuffer = (u8*)OS_MALLOC(4 * textureWidth * textureHeight);

	// Data
	if (m_textureDataBuffer != NULL)
	{
		OS_FREE(m_textureDataBuffer);
	}
	m_textureDataBuffer = (u8*)OS_MALLOC(4 * textureWidth * textureHeight);

}

// ----------------------------------------------------------------------
// CCtrlGraphics::GetTextureDataFromBmpFileData
// ----------------------------------------------------------------------
// e
//   RGB24f[^eNX`[f[^̍쐬s܂B
//   eNX`[͂Pӂȉ̃TCYłȂ΂ȂȂ
//     8, 16, 32, 64, 128, 256, 512, 1024
// ----------------------------------------------------------------------
// 
//   bmpWidth			[ I ] 摜f[^̕w肵܂B
//   bmpHeight			[ I ] 摜f[^̍w肵܂B
//   bmpDataBuffer		[ I ] 摜f[^obt@w肵܂B
//   textureWidth		[ O ] 擾eNX`[f[^̍i[܂B
//   textureHeight		[ O ] 擾eNX`[f[^̕i[܂B
//   quality			[ I ] 摜NIeBw肵܂B(1,2,4)
// ----------------------------------------------------------------------
// ߂l
//   eNX`[̃obt@AhXԂ܂B
// ----------------------------------------------------------------------
u8* CCtrlGraphics::GetTextureDataFromBmpFileData(const u32& bmpWidth, const u32& bmpHeight, u8* bmpDataBuffer, 
												u32& textureWidth, u32& textureHeight, u32 quality)
{
	int sp				= quality;
	int setBmpWidth		= bmpWidth  / sp;
	int setBmpHeight	= bmpHeight / sp;
	int addY			= sp;
	int addX			= 4 + (sp - 1) * 4;

	// Convert BMP to OpenGL RGB format
	textureWidth	= GetTextureLength(setBmpWidth);
	textureHeight	= GetTextureLength(setBmpHeight);

	// eNX`[obt@
	InitTextureBuffer(textureWidth, textureHeight);
	m_textureWidth	= textureWidth;
	m_textureHeight	= textureHeight;

	// NN_LOG("(1)graphic: bmpWidth       = %d\n", bmpWidth);
	// NN_LOG("(2)graphic: bmpHeight      = %d\n", bmpHeight);
	// NN_LOG("(3)graphic: textureWidth   = %d\n", textureWidth); 
	// NN_LOG("(4)graphic: textureHeight  = %d\n", textureHeight); 

	int sy = m_textureHeight - setBmpHeight;
	// RGB32f[^GLDataBufferBGRA`Ńf[^쐬
	// (0, 0)ƂȂ邽߁Al߂ɂ

	for (u32 y = sy; y < m_textureHeight ; y++)
	{
		u8* textureDataPtr	= m_textureGLDataBuffer + (m_textureWidth * (y)) * 3;
		u8* bmpDataPtr		= bmpDataBuffer         + (bmpWidth * (y - sy) * addY) * 4;	// RGB32

		for (u32 x = 0 ; x < setBmpWidth; x++)
		{
			*textureDataPtr++ = *(bmpDataPtr + 3);
			*textureDataPtr++ = *(bmpDataPtr + 2);
			*textureDataPtr++ = *(bmpDataPtr + 1);
			bmpDataPtr += addX;
		}
	}
 
	// Convert OpenGL RGB format to PICA Native RGB format
	bool result = demo::ConvertGLTextureToNative(GL_RGB_NATIVE_DMP, m_textureWidth, m_textureHeight, m_textureGLDataBuffer, m_textureDataBuffer);
	if (result == false)
	{
		//NN_LOG("  Convert to GL_RGB_NATIVE_DMP failed.\n");
		NN_ASSERT(0);
	}

	return m_textureDataBuffer;
}

// ----------------------------------------------------------------------
// CCtrlGraphics::DrawCamera
// ----------------------------------------------------------------------
// e
//   fBXvC1ɃeNX`[`悵܂B
// ----------------------------------------------------------------------
// 
//   textureDataPtr		[ I ] eNX`[摜f[^obt@w肵܂B
//   textureWidth		[ I ] eNX`[摜f[^̕w肵܂B
//   textureHeight		[ I ] eNX`[摜f[^̍w肵܂B
//   bmpWidth			[ I ] 摜f[^̕w肵܂B
//   bmpHeight			[ I ] 摜f[^̍w肵܂B
//   quality			[ I ] 摜f[^NIeBw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::DrawCamera(u8* textureDataPtr, u32 textureWidth, u32 textureHeight, u32 bmpWidth, u32 bmpHeight, u32 quality)
{
	static GLuint textureId = 0;
	bmpWidth  /= quality;
	bmpHeight /= quality;

	if( textureId != 0 )
	{
		m_RenderSystemDrawing.DeleteTexture(textureId);
	}

	GLenum internalFormat	= GL_RGB_NATIVE_DMP;
	GLenum format			= GL_RGB_NATIVE_DMP;
	GLenum type				= GL_UNSIGNED_BYTE;

	// eNX`[̍쐬
	m_RenderSystemDrawing.GenerateTexture(GL_TEXTURE_2D,
		internalFormat,
		textureWidth, textureHeight,
		format, type, textureDataPtr,
		textureId);

	m_RenderSystemDrawing.SetRenderTarget(NN_GX_DISPLAY1);
	m_RenderSystemDrawing.SetColor(0.0f, 0.0f, 1.0f, 0.0f);

	f32 windowPositionX = 0.0f;
	f32 windowPositionY = 0.0f;
	f32 rectangleWidth	= nn::gx::CTR::DISPLAY1_HEIGHT  * textureWidth  / bmpWidth;		// DISPLAY1_HEIGHT = 320
	f32 rectangleHeight = nn::gx::CTR::DISPLAY1_WIDTH   * textureHeight / bmpHeight;	// DISPLAY1_WIDTH  = 256

	m_RenderSystemDrawing.FillTexturedRectangle(textureId,
		windowPositionX, windowPositionY,
		rectangleWidth, rectangleHeight,
		textureWidth, textureHeight,
		textureWidth, textureHeight);
}

// ----------------------------------------------------------------------
// CCtrlGraphics::DrawDateTime
// ----------------------------------------------------------------------
// e
//   fBXvC0Ɍ݂̓tƎ`悵܂B
// ----------------------------------------------------------------------
// 
//   
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::DrawDateTime(void)
{
	// j̕f[^
	const char WeekStr[][4] = {"SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"};

	m_RenderSystemDrawing.SetRenderTarget(NN_GX_DISPLAY0);
	m_RenderSystemDrawing.SetColor(1.0f, 1.0f, 1.0f, 1.0f);

	DateTime now = DateTime::GetNow();
	m_RenderSystemDrawing.DrawText(288, 0, "%04d.%02d.%02d %s", now.GetYear(), now.GetMonth(), now.GetDay(), WeekStr[now.GetWeek()]);
	m_RenderSystemDrawing.DrawText(326, 8, "%02d:%02d:%02d", now.GetHour(), now.GetMinute(), now.GetSecond());
}

// ----------------------------------------------------------------------
// CCtrlGraphics::DrawTextEx
// ----------------------------------------------------------------------
// e
//   w肳ꂽfBXvCɕ`悵܂B
// ----------------------------------------------------------------------
// 
//   display	[ I ] fBXvCw肵܂B
//   x			[ I ] `JnXWw肵܂B
//   y			[ I ] `JnYWw肵܂B
//   fmt		[ I ] `敶tH[}bgw肵܂B
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::DrawTextEx(const s32 display, u32 x, u32 y, char* fmt, ...)
{
	// format 񏈗
	char str[128];
	va_list va;

	va_start(va, fmt);
	vsnprintf(str, 128, fmt, va);
	va_end(va);

	m_RenderSystemDrawing.SetRenderTarget(display);

	m_RenderSystemDrawing.DrawText(x, y, "%s", str);
}

// ----------------------------------------------------------------------
// CCtrlGraphics::WaitVsync
// ----------------------------------------------------------------------
// e
//   Vsync҂s܂B
// ----------------------------------------------------------------------
// 
//   
// ----------------------------------------------------------------------
// ߂l
//   
// ----------------------------------------------------------------------
void CCtrlGraphics::WaitVsync()
{
	m_RenderSystemDrawing.WaitVsync(NN_GX_DISPLAY_BOTH);
}
